import os

from polarbird.settings import settings

TOKENS_FILE = os.path.join(settings.USER_DIR, 'polarbird_tokens')
CONSUMER_KEY = 'FgSKqDOxxS4GwQ6qWFEW77mWO'
CONSUMER_SECRET = 'kGeyn7sU8MpwP3csnk6R4fKbkqCNTBhFREAJEjkMqwHcIGVFxx'
